require 'spec_helper'

describe "Veritrans checkout" do
  let!(:country) { create(:country, :states_required => true) }
  let!(:state) { create(:state, :country => country) }
  let!(:shipping_method) { create(:shipping_method) }
  let!(:stock_location) { create(:stock_location) }
  let!(:mug) { create(:product, :name => "RoR Mug") }
  let!(:stripe_payment_method) do
    Spree::Gateway::VeritransGateway.create!(
      :name => "Veritrans",
      :preferred_client_key => "VT-client-BO3Lm5WmKlzxg9FI",
      :preferred_server_key => "VT-server-HvU2kqGsVjV08_t79cN8yPiK",
      :preferred_url_api => "https://api.sandbox.veritrans.co.id"
    )
  end

  let!(:zone) { create(:zone) }

  before do
    user = create(:user)

    order = OrderWalkthrough.up_to(:delivery)
    order.stub :confirmation_required? => true

    order.reload
    order.user = user
    order.update!

    Spree::CheckoutController.any_instance.stub(:current_order => order)
    Spree::CheckoutController.any_instance.stub(:try_spree_current_user => user)
    Spree::CheckoutController.any_instance.stub(:skip_state_validation? => true)

    visit spree.checkout_state_path(:payment)
  end

  # This will pass the CC data to the server and the VeritransGateway class handles it
  it "cannot process a valid payment without tokenid" do
    fill_in "Card Number", :with => "4011 1111 1111 1112"
    fill_in "Card Code", :with => "123"
    fill_in "Expiration", :with => "01 / 20"
    click_button "Save and Continue"
    page.current_url.should include("/checkout/confirm")
    click_button "Place Order"
    page.should have_content("Token id is missing, invalid, or timed out")
  end

  # This will fetch a token from Veritrans.com and then pass that to the webserver.
  # The server then processes the payment using that token.
  it "can process a valid payment (with JS)", :js => true do
    fill_in "Card Number", :with => "4011 1111 1111 1112"
    # Otherwise ccType field does not get updated correctly
    page.execute_script("$('.cardNumber').trigger('change')")
    fill_in "Card Code", :with => "123"
    fill_in "Expiration", :with => "01 / 20"
    click_button "Save and Continue"
    sleep(5) # Wait for Veritrans API to return + form to submit
    page.current_url.should include("/checkout/confirm")
    click_button "Place Order"
    page.should have_content("Your order has been processed successfully")
  end

  it "shows an error with an invalid credit card number", :js => true do
    click_button "Save and Continue"
    page.should have_content("Invalid card_number association")
    page.should have_css('#card_number.error')
  end

  it "shows an error with invalid security fields", :js => true do
    fill_in "Card Number", :with => "4011 1111 1111 1112"
    fill_in "Expiration", :with => "01 / 20"
    click_button "Save and Continue"
    page.should have_content("card_cvv must be 3 digits")
    page.should have_content("card_cvv is required")
    page.should have_content("card_cvv does not match with regular expression")
    page.should have_css('#card_code.error')
  end

  it "shows an error with invalid expiry month field", :js => true do
    fill_in "Card Number", :with => "4011 1111 1111 1112"
    fill_in "Expiration", :with => "00 / 20"
    fill_in "Card Code", :with => "123"
    click_button "Save and Continue"
    page.should have_content("card_exp_month must be between 1 and 12")
    page.should have_css('#card_expiry.error')
  end

  it "shows an error with invalid expiry year field", :js => true do
    fill_in "Card Number", :with => "4011 1111 1111 1112"
    fill_in "Expiration", :with => "12 / "
    fill_in "Card Code", :with => "123"
    click_button "Save and Continue"
    page.should have_content("card_exp_year must be greater than this year")
    page.should have_content("card_expire_month must be greater than this year's month")
    page.should have_content("card_exp_year must be 4 digits")
    page.should have_css('#card_expiry.error')
  end
end
